/**
   it sends msg to a specified ip address and port.
   
 */

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#ifdef __MPI__
#include "mpi.h"
#endif


static int send_msg(const char* ipstr, int port, const char* msg)
{
  int s;
  struct sockaddr_in s_addr;
  int n;

  printf("addr: %s\n",ipstr);
  printf("port: %d\n",port);

  s_addr.sin_family = AF_INET;
  s_addr.sin_addr.s_addr = inet_addr( ipstr );
  s_addr.sin_port = htons(port);

  s = socket( AF_INET, SOCK_STREAM, 0);
  if(s<0) {
    printf("Unable to open socket");
    return -1;
  }

  if(connect(s,(struct sockaddr*)&s_addr, sizeof(s_addr))<0 ) {
    printf("Unable to connect");
    return -1;
  }

  n = write(s, msg, strlen(msg));
  printf("n=%d\n",n);

  close(s);

  return 0;
}


int main(int argc, char* argv[])
{
  char msgbuf[128];
  int rc;
#ifdef __MPI__
  int rank,size;
#endif

  if( argc < 4 ) {
    fprintf(stderr,"%s ip port msg\n", argv[0]);
    exit(0);
  }

#ifdef __MPI__
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);

  snprintf(msgbuf, sizeof(msgbuf), "%4d: %s\n",rank, argv[3]);

  rc = send_msg( argv[1], atoi(argv[2]), msgbuf ); 

  MPI_Finalize();
#else
  snprintf(msgbuf, sizeof(msgbuf),  "NO_RANK: %s\n",  argv[3]);

  rc = send_msg( argv[1], atoi(argv[2]),msgbuf ); 
#endif

  return rc;
}
